''*****************************************************
''*  MCP3202 12-bit/2-channel ADC Driver v1.0         *
''*  (C) 2008 Dean M goedde                           *
''*****************************************************

VAR
  long  cog
  long  ins[2]          ' 2 longs
  long  dpin, cpin, spin

PUB start(_dpin, _cpin, _spin, mode) : okay

'' Start driver - starts a cog
'' returns false if no cog available
'' may be called again to change settings
''
''   dpin  = pin connected to both DIN and DOUT on MCP3208
''   cpin  = pin connected to CLK on MCP3202
''   spin  = pin connected to CS on MCP3202
''   mode  = 0 is direct, 1 is differential

  dpin := 1 << _dpin
  cpin := 1 << _cpin
  spin := 1 << _spin

  return cog := cognew(@entry, @ins) + 1


PUB stop

'' Stop driver - frees a cog

  if cog
    cogstop(cog~ - 1)


PUB in(channel) : sample

'' Read the current sample from an ADC channel (0..1)

  return ins[channel]


DAT
'************************************
'* Assembly language MCP3208 driver *
'************************************
                        org
entry                   mov     t1,par                  'read parameters
                        mov     hub_ch0,t1              'store HUB ram location
                        add     t1,#4
                        mov     hub_ch1,t1              'store HUB ram location
                        add     t1,#4
                        rdlong  dmask,t1                'setup DIN/DOUT pin
                        add     t1,#4
                        rdlong  cmask,t1                'setup CLK pin
                        add     t1,#4
                        rdlong  smask,t1                'setup CS pin

                        or      dira,cmask              'output CLK
                        or      dira,smask              'output CS

' Perform conversions continuously
main_loop
                        mov     t0,#0                   ' Clear t0
                        mov     t1,#0                   ' Clear t1
                        mov     counter,#4              ' 4 successive measurements, later divide by 4 (shr 2)
              
iterate                 mov     stream,#%1101           ' command for ch0 read, %1101
                        call    #cloop                  ' Measure Ch0
                        add     t0,stream               ' Here Stream is the returned value
                        mov     stream,#%1111           ' command for ch1 read, %1111
                        call    #cloop                  ' Measure Ch1
                        add     t1,stream               ' Here Stream is the returned value
                        djnz    counter,#iterate

                        shr     t0,#2                   ' divide by 4 for final result
                        shr     t1,#2                   ' divide by 4 for final result
                        wrlong  t0,hub_ch0              ' write to HUB RAM
                        wrlong  t1,hub_ch1              ' write to HUB RAM

                        jmp     #main_loop              ' repeat the continuos read cycle from ADC
                                          




cloop                   andn    outa,smask              'CS low
                        or      dira,dmask              'make DIN/DOUT output
                        mov     bits,#18                'ready 18 bits (cs+1+diff+ch[1]+0+0+data[12])    MODIFIED

bloop                   test    stream,#$10     wc      'update DIN/DOUT
                        muxc    outa,dmask

                        cmp     bits,#14        wz      'if command done, input DIN/DOUT            MODIFIED
        if_z            andn    dira,dmask              'DIO now an input

                        andn    outa,cmask              'CLK low
                        nop                             'pause to give 250ns between CLK low-to-high
                        nop
                        nop
                        nop
                        or      outa,cmask              'CLK high
                        nop                             'pause to give 250ns between CLK high-to-low
                        nop
                        nop
                        nop
                        nop
                        test    dmask,ina       wc      'sample DIN/DOUT
                        rcl     stream,#1
                        djnz    bits,#bloop             'next data bit
                        or      outa,smask              'CS high
                        and     stream,mask12           'trim and write sample

cloop_ret               ret


' Initialized data
mask12                  long    $FFF
' Uninitialized data
hub_ch0                 res     1
hub_ch1                 res     1
t0                      res     1
t1                      res     1
dmask                   res     1
cmask                   res     1
smask                   res     1
stream                  res     1
bits                    res     1
counter                 res     1